/* ---------------------------------------------------------------------------------------------------- 
   gDraw picture converter 0.2	(Dez. 15. 1998)

   Copyright (c) by Collin R. Mulliner <collin@mulliner.org>
---------------------------------------------------------------------------------------------------- */

#include <stdio.h>
#include "cp_1.h"
#include "help.h"

/* ------------------------------------------------------------------------------------------------- */
void write_image_as_pcx_file(int pic_number, char f_name[], int b_read);
int print_image_names(char f_name[], int *b_read);
void get_4pixels(unsigned char the_pixels, unsigned char pixels_4[]);
void write_pcx_header(FILE *pcx_fp);
void write_color_palette(FILE *pcx_fp);
/* ------------------------------------------------------------------------------------------------- */
#define MASK_1 0xC0
#define MASK_2 0x30
#define MASK_3 0x0C
#define MASK_4 0x03

#define COLOR_1 0xC0
#define COLOR_2 0x80
#define COLOR_3 0x40
#define COLOR_4 0x00

#define PCX_COLOR_1 128
// for COLOR_1
#define PCX_COLOR_2 64
// for COLOR_2
#define PCX_COLOR_3 0
// for COLOR_3
#define PCX_COLOR_4 255
// for COLOR_4
/* ------------------------------------------------------------------------------------------------- */
void main(int argc, char *argv[])
{
	int image_number, i, read_before;

	if (argc == 1) { printf(HELP_MSG); return; }

	printf("converting ...\n");
	image_number = print_image_names(argv[1], &read_before);

	for (i = 0; i < image_number; i++) {
		write_image_as_pcx_file(i, argv[1], read_before);
	}
	printf("... done\n");
}
/* ------------------------------------------------------------------------------------------------- */
void write_image_as_pcx_file(int pic_number, char f_name[], int b_read)
{
	FILE *fp, *fp2;
	int i, i2, pc;
	unsigned char pbyte;
	unsigned char pixles[4];
	char p_name[30];
	char ok;

	fp = fopen(f_name, "r");

	for (i = 0; i < b_read; i++) { fgetc(fp); }	// read *.pdb header
	
	for (pc = 0; pc <= pic_number - 1; pc++) {
		for (i = 0; i < 6420; i++) { fgetc(fp); }	// read the image data
	}
	
	for (i = 0; i < 20; i++) {
		p_name[i] = fgetc(fp);			// read the picture name
	}



	strcat(p_name, ".pcx");				// add the file suffix "pcx" to picturename
	fp2 = fopen(p_name, "wb");

//	this is for Linux/U*ix
/*
	p_name[8] = '\0';
	strcat(p_name, ".pcx");				// add the file suffix "pcx" to picturename

	ok = '\x30';
	while (ok != 0) {
		fp2 = fopen(p_name, "rb");
		if (fp2 != NULL) {
			fclose(fp2);
			p_name[7] = ok;
			ok++;
		}
		else { ok = 0; }
	}

	fp2 = fopen(p_name, "wb");
*/
//	this is for DOS/Win

	write_pcx_header(fp2);

	for (i = 0; i < 6400; i++) {
		pbyte = fgetc(fp);
		get_4pixels(pbyte, pixles);
		for (i2 = 0; i2 < 4; i2++) {
			fputc('\xC1', fp2);
			fputc(pixles[i2], fp2);
		}
	}	

	write_color_palette(fp2);
	
	fclose(fp);
	fclose(fp2);
}
/* ------------------------------------------------------------------------------------------------- */
int print_image_names(char f_name[], int *b_read)
{
	FILE *fp;
	int i, file_ok;
	char p_name[21];
	int p_count;
	char c;
	int ok, at_c;

	fp = fopen(f_name, "r");

	for (i = 0; i < 83+41; i++) {
		c = fgetc(fp);
	}
	
	at_c = 1;
	ok = 1;

	while (ok == 1) {
		for (i = 0; i < 8; i++) { c = fgetc(fp); }
		if (c == '@') { at_c++; }
		else { ok = 0; }
	}
	for (i = 0; i < 18; i++) { c = fgetc(fp); }

	*(b_read) = 83+41 + (at_c * 8) + 18;
	
	file_ok = 1;
	p_count = 0;

	while (file_ok == 1) {
		p_name[0] = fgetc(fp);

		if (p_name[0] != EOF) { 
			for (i = 1; i < 20; i++) {
				p_name[i] = fgetc(fp);			// read the picture name
			}
			p_count++;
			printf("%s\n",p_name);
			for (i = 0; i < 6400; i++) { fgetc(fp); }	// read the image data
		}
		else { file_ok = 0; }	
	}

	fclose(fp);
	return p_count;	
}
/* ------------------------------------------------------------------------------------------------- */
void get_4pixels(unsigned char the_pixels, unsigned char pixels_4[])
{
	int no;
	unsigned char new_pixels;

	for (no = 1; no < 5; no++) {
		if (no == 1) { 
			new_pixels = the_pixels & MASK_1;
		}
		else if (no == 2) { 
			new_pixels = the_pixels & MASK_2;
			new_pixels = new_pixels << 2;
		}
		else if (no == 3) { 
			new_pixels = the_pixels & MASK_3;
			new_pixels = new_pixels << 4;
		}
		else if (no == 4) { 
			new_pixels = the_pixels & MASK_4;
			new_pixels = new_pixels << 6;
		}

		if (new_pixels == COLOR_1) { 
			pixels_4[no - 1] = PCX_COLOR_1;
		}
		else if (new_pixels == COLOR_2) { 
			pixels_4[no - 1] = PCX_COLOR_2;
		}
		else if (new_pixels == COLOR_3) { 
			pixels_4[no - 1] = PCX_COLOR_3;
		}
		else if (new_pixels == COLOR_4) { 
			pixels_4[no - 1] = PCX_COLOR_4;
		}
	}
}
/* ------------------------------------------------------------------------------------------------- */
void write_pcx_header(FILE *pcx_fp)
{
	int i;

	fputc('\x0A', pcx_fp);		// PCX ID 0x0A
	fputc('\x05', pcx_fp);		// version -> 0x05 = version 3.0
	fputc('\x01', pcx_fp);		// RLE compression YES
	fputc('\x08', pcx_fp);		// color depth -> 8 bit
	fputc('\x00', pcx_fp);		// upper left X
	fputc('\x00', pcx_fp);		// upper left X
	fputc('\x00', pcx_fp);		// upper left Y
	fputc('\x00', pcx_fp);		// upper left Y
	fputc('\x9f', pcx_fp);		// lower right X
	fputc('\x00', pcx_fp);		// lower right X
	fputc('\x9f', pcx_fp);		// lower right Y 
	fputc('\x00', pcx_fp);		// lower right Y
	fputc('\x2C', pcx_fp);		// resulution 300 DPI  
	fputc('\x01', pcx_fp);		// resulution 300 DPI  
	fputc('\x2C', pcx_fp);		// resulution 300 DPI  
	fputc('\x01', pcx_fp);		// resulution 300 DPI  

	for (i = 1; i < 49; i++) {			// write 48 bytes
		fputc(color_palette[i], pcx_fp);	// write first 16 colors from the color palette
	}					

	fputc('\x00', pcx_fp);		// reserved byte
	fputc('\x01', pcx_fp);		// number of planes -> 1
	fputc('\xA0', pcx_fp);		// bytes per line -> 160 
	fputc('\x00', pcx_fp);		// bytes per line -> 160
	fputc('\x02', pcx_fp);		// palette type -> 2 = grayscale (255 grays) 
	fputc('\x00', pcx_fp);		// palette type -> 2 = grayscale (255 grays) 
	
	for (i = 0; i < 58; i++) {
		fputc('\x00', pcx_fp);			// fill header to 128 bytes
	}
}
/* ------------------------------------------------------------------------------------------------- */
void write_color_palette(FILE *pcx_fp)
{
	int i;

	for (i = 0; i < 769; i++) {
		fputc(color_palette[i], pcx_fp);
	}
}
/* ------------------------------------------------------------------------------------------------- */

